# This file is part of Supysonic.
# Supysonic is a Python implementation of the Subsonic server API.
#
# Copyright (C) 2013-2018 Alban 'spl0k' Féron
#                    2017 Óscar García Amor
#
# Distributed under terms of the GNU AGPLv3 license.

NAME = "Supysonic"
VERSION = "0.7.8"
DESCRIPTION = "Python implementation of the Subsonic server API"
AUTHOR = "Alban Féron"
AUTHOR_EMAIL = "alban.feron@gmail.com"
URL = "https://supysonic.readthedocs.io/"
DOWNLOAD_URL = "https://github.com/spl0k/supysonic"
LICENSE = "GNU AGPLv3"
