{#-
 This file is part of Supysonic.
 Supysonic is a Python implementation of the Subsonic server API.

 Copyright (C) 2013-2017 Alban 'spl0k' Féron
                    2017 Óscar García Amor

 Distributed under terms of the GNU AGPLv3 license.
-#}
{% extends "layout.html" %}
{% block body %}
<div id="loginbox" class="mainbox col-md-6 col-md-offset-3 col-sm-6 col-sm-offset-3">
  <div class="panel panel-default" >
    <div class="panel-heading">
      <div class="panel-title text-center">Log in</div>
    </div>
    <div class="panel-body" >
      <form id="loginform" method="post">
        <div class="form-group">
          <label class="sr-only" for="user">User</label>
          <div class="input-group">
            <div class="input-group-addon"><span class="glyphicon glyphicon-user"></span></div>
            <input type="text" class="form-control" id="user" name="user" value="{{ request.form.user }}" placeholder="User" />
          </div>
        </div>
        <div class="form-group">
          <label class="sr-only" for="password">Password</label>
          <div class="input-group">
            <div class="input-group-addon"><span class="glyphicon glyphicon-lock"></span></div>
            <input type="password" class="form-control" id="password" name="password" placeholder="Password" />
          </div>
        </div>
        <div class="form-group">
          <!-- Button -->
          <div class="push-left">
            <button type="submit" class="btn btn-primary pull-right"><i class="glyphicon glyphicon-log-in"></i> Log in</button>
          </div>
        </div>
      </form>
    </div>
  </div>
</div>
{% endblock %}
