{#-
 This file is part of Supysonic.
 Supysonic is a Python implementation of the Subsonic server API.

 Copyright (C) 2013-2018 Alban 'spl0k' Féron
                    2017 Óscar García Amor

 Distributed under terms of the GNU AGPLv3 license.
-#}
{% extends "layout.html" %}
{% block navbar_folders %}
<li class="active"><a href="{{ url_for('frontend.folder_index') }}">Folders <span
                      class="sr-only">(current)</span></a></li>
{% endblock %}
{% block body %}
<div class="page-header first-header">
  <h2>Add Folder</h2>
</div>
<form method="post">
  <div class="form-group">
    <label class="sr-only" for="user">Name</label>
    <div class="input-group">
      <div class="input-group-addon"><span class="glyphicon glyphicon-pencil"></span></div>
      <input type="text" class="form-control" id="name" name="name" value="{{ request.form.name }}" placeholder="Name" />
    </div>
  </div>
  <div class="form-group">
    <label class="sr-only" for="user">Path</label>
    <div class="input-group">
      <div class="input-group-addon"><span class="glyphicon glyphicon-folder-open"></span></div>
      <input type="text" class="form-control" id="path" name="path" value="{{ request.form.path }}" placeholder="Path" />
    </div>
  </div>
  <input type="submit" class="btn btn-default" />
</form>
{% endblock %}
